// Copyright (C) 2019 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#ifndef COMMONDEFS_H
#define COMMONDEFS_H

#define SERVICE_NAME "x11vkb.chromium.dbus.service"

#endif // COMMONDEFS_H
