// Copyright (C) 2017 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only

#include <QtVirtualKeyboard/private/gesturerecognizer_p.h>

QT_BEGIN_NAMESPACE
namespace QtVirtualKeyboard {

GestureRecognizer::GestureRecognizer(QObject *parent) :
    QObject(parent)
{
}

} // namespace QtVirtualKeyboard
QT_END_NAMESPACE
